@@ -1,14 +1,23 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from stdnum import iban, bic
 import stdnum.exceptions
-from sql import operators, Literal
+from sql import Literal, operators
+from sql.operators import Equal
+from stdnum import bic, iban
+
+try:
+    from schwifty import BIC, IBAN
+except ImportError:
+    BIC = IBAN = None
 
 from trytond.i18n import gettext
 from trytond.model import (
-    ModelView, ModelSQL, DeactivableMixin, fields, sequence_ordered)
+    DeactivableMixin, Exclude, ModelSQL, ModelView, fields, sequence_ordered)
+from trytond.pool import Pool
+from trytond.pyson import Eval, If
+from trytond.tools import is_full_text, lstrip_wildcard
 
-from .exceptions import IBANValidationError, InvalidBIC
+from .exceptions import AccountValidationError, IBANValidationError, InvalidBIC
 
 
 class Bank(ModelSQL, ModelView):
@@ -23,7 +32,18 @@ class Bank(ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        return [('party.rec_name',) + tuple(clause[1:])]
+        _, operator, operand, *extra = clause
+        if operator.startswith('!') or operator.startswith('not '):
+            bool_op = 'AND'
+        else:
+            bool_op = 'OR'
+        bic_value = operand
+        if operator.endswith('like') and is_full_text(operand):
+            bic_value = lstrip_wildcard(operand)
+        return [bool_op,
+            ('party.rec_name', operator, operand, *extra),
+            ('bic', operator, bic_value, *extra),
+            ]
 
     @fields.depends('bic')
     def on_change_with_bic(self):
@@ -42,24 +62,55 @@ class Bank(ModelSQL, ModelView):
         if self.bic and not bic.is_valid(self.bic):
             raise InvalidBIC(gettext('bank.msg_invalid_bic', bic=self.bic))
 
+    @classmethod
+    def from_bic(cls, bic):
+        "Return or create bank from BIC instance"
+        pool = Pool()
+        Party = pool.get('party.party')
+        if IBAN:
+            assert isinstance(bic, BIC)
+            banks = cls.search([
+                    ('bic', '=', bic.compact),
+                    ], limit=1)
+            if banks:
+                bank, = banks
+                return bank
+            cls.lock()
+            names = bic.bank_names
+            if names:
+                name = names[0]
+            else:
+                name = None
+            bank = cls(party=Party(name=name), bic=bic.compact)
+            bank.save()
+            return bank
+
 
 class Account(DeactivableMixin, ModelSQL, ModelView):
     'Bank Account'
     __name__ = 'bank.account'
-    bank = fields.Many2One('bank', 'Bank', required=True,
+    bank = fields.Many2One(
+        'bank', "Bank",
         help="The bank where the account is open.")
     owners = fields.Many2Many('bank.account-party.party', 'account', 'owner',
         'Owners')
     currency = fields.Many2One('currency.currency', 'Currency')
-    numbers = fields.One2Many('bank.account.number', 'account', 'Numbers',
+    numbers = fields.One2Many(
+        'bank.account.number', 'account', 'Numbers',
+        domain=[
+            If(~Eval('active'), ('active', '=', False), ()),
+            ],
         help="Add the numbers which identify the bank account.")
 
     def get_rec_name(self, name):
-        if self.numbers:
-            number = self.numbers[0].number
+        for number in self.numbers:
+            if number.number:
+                name = number.number
+                break
         else:
-            number = '(%s)' % self.id
-        name = '%s @ %s' % (number, self.bank.rec_name)
+            name = '(%s)' % self.id
+        if self.bank:
+            name += ' @ %s' % self.bank.rec_name
         if self.currency:
             name += ' [%s]' % self.currency.code
         return name
@@ -76,13 +127,64 @@ class Account(DeactivableMixin, ModelSQL, ModelView):
             ('numbers.rec_name',) + tuple(clause[1:]),
             ]
 
+    @property
+    def iban(self):
+        for number in self.numbers:
+            if number.type == 'iban':
+                return number.number
 
-class AccountNumber(sequence_ordered(), ModelSQL, ModelView):
+    @classmethod
+    def validate(cls, accounts):
+        super().validate(accounts)
+        for account in accounts:
+            account.check_bank()
+
+    def check_bank(self):
+        if not self.bank or not self.bank.bic:
+            return
+        if IBAN and BIC and self.iban:
+            iban = IBAN(self.iban)
+            bic = BIC(self.bank.bic)
+            if (iban.bic
+                    and iban.bic != bic
+                    and (
+                        iban.country_code != bic.country_code
+                        or (iban.bank_code or iban.branch_code)
+                        not in bic.domestic_bank_codes)):
+                raise AccountValidationError(
+                    gettext('bank.msg_account_iban_invalid_bic',
+                        account=self.rec_name,
+                        bic=iban.bic))
+
+    @classmethod
+    def create(cls, vlist):
+        accounts = super().create(vlist)
+        for account in accounts:
+            if not account.bank:
+                bank = account.guess_bank()
+                if bank:
+                    account.bank = bank
+        cls.save(accounts)
+        return accounts
+
+    def guess_bank(self):
+        pool = Pool()
+        Bank = pool.get('bank')
+        if IBAN and self.iban:
+            iban = IBAN(self.iban)
+            if iban.bic:
+                return Bank.from_bic(iban.bic)
+
+
+class AccountNumber(DeactivableMixin, sequence_ordered(), ModelSQL, ModelView):
     'Bank Account Number'
     __name__ = 'bank.account.number'
     _rec_name = 'number'
-    account = fields.Many2One('bank.account', 'Account', required=True,
-        ondelete='CASCADE', select=True,
+    account = fields.Many2One(
+        'bank.account', "Account", required=True, ondelete='CASCADE',
+        domain=[
+            If(Eval('active'), ('active', '=', True), ()),
+            ],
         help="The bank account which is identified by the number.")
     type = fields.Selection([
             ('iban', 'IBAN'),
@@ -93,7 +195,20 @@ class AccountNumber(sequence_ordered(), ModelSQL, ModelView):
 
     @classmethod
     def __setup__(cls):
+        cls.number.search_unaccented = False
+        cls.number_compact.search_unaccented = False
         super().__setup__()
+        table = cls.__table__()
+        cls._sql_constraints += [
+            ('number_iban_exclude',
+                Exclude(table, (table.number_compact, Equal),
+                    where=table.type == 'iban'),
+                'bank.msg_number_iban_unique'),
+            ('account_iban_exclude',
+                Exclude(table, (table.account, Equal),
+                    where=table.type == 'iban'),
+                'bank.msg_account_iban_unique'),
+            ]
         cls.__access__.add('account')
         cls._order.insert(0, ('account', 'ASC'))
 
@@ -128,6 +243,20 @@ class AccountNumber(sequence_ordered(), ModelSQL, ModelView):
         return (iban.compact(self.number) if self.type == 'iban'
             else self.number)
 
+    @classmethod
+    def search_rec_name(cls, name, clause):
+        _, operator, operand, *extra = clause
+        if operator.startswith('!') or operator.startswith('not '):
+            bool_op = 'AND'
+        else:
+            bool_op = 'OR'
+        number_value = operand
+        if operator.endswith('like') and is_full_text(operand):
+            number_value = lstrip_wildcard(operand)
+        return [bool_op,
+            ('number', operator, number_value, *extra),
+            ]
+
     @classmethod
     def create(cls, vlist):
         vlist = [v.copy() for v in vlist]
@@ -177,7 +306,7 @@ class AccountNumber(sequence_ordered(), ModelSQL, ModelView):
 class AccountParty(ModelSQL):
     'Bank Account - Party'
     __name__ = 'bank.account-party.party'
-    account = fields.Many2One('bank.account', 'Account',
-        ondelete='CASCADE', select=True, required=True)
-    owner = fields.Many2One('party.party', 'Owner', ondelete='CASCADE',
-        select=True, required=True)
+    account = fields.Many2One(
+        'bank.account', "Account", ondelete='CASCADE', required=True)
+    owner = fields.Many2One(
+        'party.party', "Owner", ondelete='CASCADE', required=True)
