@@ -1,8 +1,8 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from trytond.pool import PoolMeta
 from trytond.model import fields
-from trytond.tools import lstrip_wildcard
+from trytond.pool import PoolMeta
+from trytond.transaction import Transaction
 
 
 class Party(metaclass=PoolMeta):
@@ -18,15 +18,22 @@ class Party(metaclass=PoolMeta):
             bool_op = 'AND'
         else:
             bool_op = 'OR'
-        code_value = clause[2]
-        if clause[1].endswith('like'):
-            code_value = lstrip_wildcard(clause[2])
         return [bool_op,
             domain,
-            ('bank_accounts.numbers.rec_name',
-                clause[1], code_value) + tuple(clause[3:]),
+            ('bank_accounts.numbers.rec_name',) + tuple(clause[1:]),
             ]
 
+    @classmethod
+    def copy(cls, parties, default=None):
+        default = default.copy() if default else {}
+        if Transaction().check_access:
+            default.setdefault(
+                'bank_accounts',
+                cls.default_get(
+                    ['bank_accounts'],
+                    with_rec_name=False).get('bank_accounts'))
+        return super().copy(parties, default=default)
+
 
 class Replace(metaclass=PoolMeta):
     __name__ = 'party.replace'
